summaryrefslogtreecommitdiff
path: root/lisp/ielm.el
diff options
context:
space:
mode:
Diffstat (limited to 'lisp/ielm.el')
-rw-r--r--lisp/ielm.el107
1 files changed, 77 insertions, 30 deletions
diff --git a/lisp/ielm.el b/lisp/ielm.el
index aa60d5de6c3..944e2453cb9 100644
--- a/lisp/ielm.el
+++ b/lisp/ielm.el
@@ -1,6 +1,6 @@
;;; ielm.el --- interaction mode for Emacs Lisp
-;; Copyright (C) 1994, 2002, 2003 Free Software Foundation, Inc.
+;; Copyright (C) 1994, 2002, 2003, 2004 Free Software Foundation, Inc.
;; Author: David Smith <maa036@lancaster.ac.uk>
;; Maintainer: FSF
@@ -49,12 +49,57 @@
:type 'boolean
:group 'ielm)
+(defcustom ielm-prompt-read-only t
+ "If non-nil, the IELM prompt is read only.
+The read only region includes the newline before the prompt.
+Setting this variable does not affect existing IELM runs.
+This works by setting the buffer-local value of `comint-prompt-read-only'.
+Setting that value directly affects new prompts in the current buffer.
+
+If this option is enabled, then the safe way to temporarily
+override the read-only-ness of ielm prompts is to call
+`comint-kill-whole-line' or `comint-kill-region' with no
+narrowing in effect. This way you will be certain that none of
+the remaining prompts will be accidentally messed up. You may
+wish to put something like the following in your `.emacs' file:
+
+\(add-hook 'ielm-mode-hook
+ '(lambda ()
+ (define-key ielm-map \"\C-w\" 'comint-kill-region)
+ (define-key ielm-map [C-S-backspace]
+ 'comint-kill-whole-line)))
+
+If you set `comint-prompt-read-only' to t, you might wish to use
+`comint-mode-hook' and `comint-mode-map' instead of
+`ielm-mode-hook' and `ielm-map'. That will affect all comint
+buffers, including ielm buffers. If you sometimes use ielm on
+text-only terminals or with `emacs -nw', you might wish to use
+another binding for `comint-kill-whole-line'."
+ :type 'boolean
+ :group 'ielm
+ :version "21.4")
+
(defcustom ielm-prompt "ELISP> "
- "Prompt used in IELM."
+ "Prompt used in IELM.
+Setting this variable does not affect existing IELM runs.
+
+Interrupting the IELM process with \\<ielm-map>\\[comint-interrupt-subjob],
+and then restarting it using \\[ielm], makes the then current
+default value affect _new_ prompts. Unless the new prompt
+differs only in text properties from the old one, IELM will no
+longer recognize the old prompts. However, executing \\[ielm]
+does not update the prompt of an *ielm* buffer with a running process.
+For IELM buffers that are not called `*ielm*', you can execute
+\\[inferior-emacs-lisp-mode] in that IELM buffer to update the value,
+for new prompts. This works even if the buffer has a running process."
:type 'string
- :group 'ielm
- :get #'(lambda (symbol) (substring-no-properties (symbol-value symbol)))
- :set #'(lambda (symbol value) (set symbol (propertize value 'read-only t 'rear-nonsticky t))))
+ :group 'ielm)
+
+(defvar ielm-prompt-internal "ELISP> "
+ "Stored value of `ielm-prompt' in the current IELM buffer.
+This is an internal variable used by IELM. Its purpose is to
+prevent a running IELM process from being messed up when the user
+customizes `ielm-prompt'.")
(defcustom ielm-dynamic-return t
"*Controls whether \\<ielm-map>\\[ielm-return] has intelligent behaviour in IELM.
@@ -145,9 +190,7 @@ This variable is buffer-local.")
(define-key ielm-map "\C-c\C-v" 'ielm-print-working-buffer))
(defvar ielm-font-lock-keywords
- (list
- (cons (concat "^" (regexp-quote ielm-prompt)) 'font-lock-keyword-face)
- '("\\(^\\*\\*\\*[^*]+\\*\\*\\*\\)\\(.*$\\)"
+ '(("\\(^\\*\\*\\*[^*]+\\*\\*\\*\\)\\(.*$\\)"
(1 font-lock-comment-face)
(2 font-lock-constant-face)))
"Additional expressions to highlight in ielm buffers.")
@@ -250,8 +293,7 @@ simply inserts a newline."
(defun ielm-send-input nil
"Evaluate the Emacs Lisp expression after the prompt."
(interactive)
- (let ((buf (current-buffer))
- ielm-input) ; set by ielm-input-sender
+ (let (ielm-input) ; set by ielm-input-sender
(comint-send-input) ; update history, markers etc.
(ielm-eval-input ielm-input)))
@@ -374,7 +416,7 @@ simply inserts a newline."
(setq ** *)
(setq * ielm-result))
(setq ielm-output (concat ielm-output "\n"))))
- (setq ielm-output (concat ielm-output ielm-prompt))
+ (setq ielm-output (concat ielm-output ielm-prompt-internal))
(comint-output-filter (ielm-process) ielm-output)))
;;; Process and marker utilities
@@ -414,8 +456,8 @@ The current working buffer may be changed (with a call to
`set-buffer', or with \\[ielm-change-working-buffer]), and its value
is preserved between successive evaluations. In this way, expressions
may be evaluated in a different buffer than the *ielm* buffer.
-Display the name of the working buffer with \\[ielm-print-working-buffer],
-or the buffer itself with \\[ielm-display-working-buffer].
+By default, its name is shown on the mode line; you can always display
+it with \\[ielm-print-working-buffer], or the buffer itself with \\[ielm-display-working-buffer].
During evaluations, the values of the variables `*', `**', and `***'
are the results of the previous, second previous and third previous
@@ -426,14 +468,16 @@ buffer, then the values in the working buffer are used. The variables
Expressions evaluated by IELM are not subject to `debug-on-quit' or
`debug-on-error'.
-The behaviour of IELM may be customised with the following variables:
-* To stop beeping on error, set `ielm-noisy' to nil
+The behaviour of IELM may be customized with the following variables:
+* To stop beeping on error, set `ielm-noisy' to nil.
* If you don't like the prompt, you can change it by setting `ielm-prompt'.
-* Set `ielm-dynamic-return' to nil for bindings like `lisp-interaction-mode'
+* If you do not like that the prompt is (by default) read-only, set
+ `ielm-prompt-read-only' to nil.
+* Set `ielm-dynamic-return' to nil for bindings like `lisp-interaction-mode'.
* Entry to this mode runs `comint-mode-hook' and `ielm-mode-hook'
(in that order).
-Customised bindings may be defined in `ielm-map', which currently contains:
+Customized bindings may be defined in `ielm-map', which currently contains:
\\{ielm-map}"
(interactive)
(comint-mode)
@@ -443,15 +487,16 @@ Customised bindings may be defined in `ielm-map', which currently contains:
(setq comint-input-sender 'ielm-input-sender)
(setq comint-process-echoes nil)
(make-local-variable 'comint-dynamic-complete-functions)
+ (set (make-local-variable 'ielm-prompt-internal) ielm-prompt)
+ (set (make-local-variable 'comint-prompt-read-only) ielm-prompt-read-only)
(setq comint-dynamic-complete-functions
'(ielm-tab comint-replace-by-expanded-history ielm-complete-filename ielm-complete-symbol))
(setq comint-get-old-input 'ielm-get-old-input)
(make-local-variable 'comint-completion-addsuffix)
- (setq comint-completion-addsuffix
- (cons (char-to-string directory-sep-char) ""))
-
+ (setq comint-completion-addsuffix '("/" . ""))
(setq major-mode 'inferior-emacs-lisp-mode)
(setq mode-name "IELM")
+ (setq mode-line-process '(":%s on " (:eval (buffer-name ielm-working-buffer))))
(use-local-map ielm-map)
(set-syntax-table emacs-lisp-mode-syntax-table)
@@ -494,10 +539,11 @@ Customised bindings may be defined in `ielm-map', which currently contains:
(insert ielm-header)
(ielm-set-pm (point-max))
(unless comint-use-prompt-regexp-instead-of-fields
- (add-text-properties
- (point-min) (point-max)
- '(rear-nonsticky t field output inhibit-line-move-field-capture t)))
- (comint-output-filter (ielm-process) ielm-prompt)
+ (let ((inhibit-read-only t))
+ (add-text-properties
+ (point-min) (point-max)
+ '(rear-nonsticky t field output inhibit-line-move-field-capture t))))
+ (comint-output-filter (ielm-process) ielm-prompt-internal)
(set-marker comint-last-input-start (ielm-pm))
(set-process-filter (get-buffer-process (current-buffer)) 'comint-output-filter))
@@ -521,12 +567,13 @@ Customised bindings may be defined in `ielm-map', which currently contains:
"Interactively evaluate Emacs Lisp expressions.
Switches to the buffer `*ielm*', or creates it if it does not exist."
(interactive)
- (if (comint-check-proc "*ielm*")
- nil
- (save-excursion
- (set-buffer (get-buffer-create "*ielm*"))
- (inferior-emacs-lisp-mode)))
- (pop-to-buffer "*ielm*"))
+ (let (old-point)
+ (unless (comint-check-proc "*ielm*")
+ (with-current-buffer (get-buffer-create "*ielm*")
+ (unless (zerop (buffer-size)) (setq old-point (point)))
+ (inferior-emacs-lisp-mode)))
+ (pop-to-buffer "*ielm*")
+ (when old-point (push-mark old-point))))
(provide 'ielm)